//********************************************************************
//*** ResourcePool/Execute.hh
//*** Copyright (c) 2003-2008 by Markus Winand <mws@fatalmind.com>
//*** $Id: Execute.cc,v 1.16 2008/05/18 12:21:36 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include <iostream>

#include "Execute.hh"

#ifndef RESOURCEPOOL_CREATEEXCEPTION_HH
#include "CreateException.hh"
#endif

#ifndef RESOURCEPOOL_NOFAILOVEREXCEPTION_HH
#include "NoFailoverException.hh"
#endif

namespace fatalmind {

template<class Type>
_RPExecute<Type>::_RPExecute()
{
}

template<class Type>
void _RPExecute<Type>::execute(Command<Type>& cmd)
{
	int trynr = getOptions().getMaxExecTry() - 1;
	bool success = false;
	do {
		typename Type::resource_t& rec = get();
		try {
			cmd.topLevel(true);
			cmd.execute(rec);
			this->free(rec);
			success=true;
		} catch (const NoFailoverException& e) {
			this->free(rec);
			throw;
		} catch (...) {
//			std::cout << "got Exception, will failover" << std::endl;
			fail(rec);
			if (trynr-- <= 0) {
				throw;
			}
		}
	} while (! success);
}

} // namespace fatalmind
