//********************************************************************
//*** ResourcePool/SQL/RTSQLBatch.cc
//*** Copyright (c) 2003-2008 by Markus Winand <mws@fatalmind.com>
//*** $Id: RTSQLBatch.cc,v 1.2 2008/04/14 07:43:53 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include "config.h"
#include "ResourcePool/SQL/ResourcePool.hh"
#include "ResourcePool/SQL/RTSQLBatch.hh"
#include "ResourcePool/SQL/RTSQLBatch_Impl.hh"

#include "ResourcePool/SQL/ConcreteWrapper.hh"

#include "ResourcePool/mysql/ResourcePool.hh"
#include "ResourcePool/oracle/ResourcePool.hh"

#include <sstream>
#include <typeinfo>

namespace fatalmind {
namespace SQL {

template<class PT, class TM, class RTSQLC>
class SQLBatchWrapper
: public PT::command_t::SQLBatch
{
		typedef typename PT::command_t::SQLBatch super;
    public:
		SQLBatchWrapper(RTSQLC& rtc, const bool cont)
		: super(cont)
		{
		}
};

template<class TM>
struct SQLBatchCommandSelector {
	typedef RTSQLBatch_Impl<TM> Impl;
	typedef RTSQLBatch<TM> RTC;

	template<class PT>
	struct Specialized {
		typedef RTSQLBatch_SpecializedImpl<
			  TM
			, RTSQLBatch_Impl
			, PT
			, SQLBatchWrapper<PT, TM, RTSQLBatch<TM> >
			, RTSQLBatch<TM>
		> Impl;
	};
};

template<class TM>
RTSQLBatch<TM>::RTSQLBatch(const ResourcePool<ResourceType<TM> >& p, const bool cont)
: super(p, cont)
, dbType(innercommand_t::getFactory(p).getDatabaseType())
, _realSQLBatch(
	RTSQLBatch_Impl<TM>::template newRealCommand<
		SQLBatchCommandSelector<TM>
	>(
		  dbType
		, *this
		, cont
	)
)
{
}

template<class TM>
RTSQLBatch<TM>::RTSQLBatch(const RTSQLBatch<TM>& rho) 
: super(rho)
, dbType(rho.dbType)
, _realSQLBatch(rho._realSQLBatch->template clone<RTSQLBatch_Impl<TM> >())
{
}

template<class TM>
RTSQLBatch<TM>::~RTSQLBatch() {
}

template<class TM>
Clone* RTSQLBatch<TM>::DoClone() const {
	return new RTSQLBatch<TM>(*this);
}


template<class TM>
void RTSQLBatch<TM>::execute(typename ResourceType<TM>::resource_t& hand) {
	_realSQLBatch->execute(hand);
}

template<class TM>
int RTSQLBatch<TM>::getError() {
	return _realSQLBatch->getError();
}

template<class TM>
typename RTSQLBatch<TM>::const_iterator RTSQLBatch<TM>::getExecPosition() const {
	const_iterator rv = super::begin();
	int off = _realSQLBatch->getExecOffset();
	if (off < 0 ) {
		rv = super::end();
	} else {
		std::advance(rv, off);
	}
	return rv;
}

template<class TM>
typename RTSQLBatch<TM>::const_iterator RTSQLBatch<TM>::begin() const {
	return super::begin();
}

template<class TM>
typename RTSQLBatch<TM>::const_iterator RTSQLBatch<TM>::end() const {
	return super::end();
}

template<class TM>
bool RTSQLBatch<TM>::doesContinue() const {
	return _realSQLBatch->doesContinue();
}

template<class TM>
bool RTSQLBatch<TM>::empty() const {
	return _realSQLBatch->empty();
}

template<class TM>
void RTSQLBatch<TM>::outputoperator(std::ostream& s) const {
	s << "(RT)";	
	_realSQLBatch->outputoperator(s);
}



template<class TM>
void RTSQLBatch<TM>::add(innercommand_t const& cmd) {
	super::add(cmd);
	_realSQLBatch->add(cmd);
}

template class RTSQLBatch<DefaultThreadedModel >;

} // namespace SQL
} // namespace fatalmind
