//********************************************************************
//*** ResourcePool/oracle/OptimizedBatchCommand.hh
//*** Copyright (c) 2003-2008 by Markus Winand <mws@fatalmind.com>
//*** $Id: OptimizedBatchCommand.hh,v 1.6 2008/03/02 19:33:47 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_ORACLE_OPTIMIZEDBATCHCOMMAND_HH
#define RESOURCEPOOL_ORACLE_OPTIMIZEDBATCHCOMMAND_HH

#include "ResourcePool/BatchCommand.hh"
#include "SQLExecute.hh"

namespace fatalmind {
namespace oracle {

/**
 * The class is a collection of Commands to be executed in sequential order.
 * Execution of commands stopps in case an exception is thrown.
 * BatchCommand's can also be nested.
 */
template<class CommandType>
class OptimizedBatchCommand: public BatchCommand<CommandType>
{
	protected:
		typedef  BatchCommand<CommandType> super;
		typedef  SQLExecute<typename CommandType::pool_type::ThreadingModell,  oracle::SelectListMetaDataCache> optimizeablecommand_t;
	public:
		OptimizedBatchCommand(const OptimizedBatchCommand& rho);
		OptimizedBatchCommand(bool a_cont = false);
		OptimizedBatchCommand(const ResourcePool<typename CommandType::pool_type >& p, bool a_cont = false);

		virtual void add(const CommandType&);
		virtual void add(const optimizeablecommand_t&);
		virtual void execute(typename CommandType::pool_type::resource_t&);

		virtual void outputoperator(std::ostream&) const;
	protected:
		virtual Clone* DoClone() const {
			return new OptimizedBatchCommand(*this);
		}
	private:
		optimizeablecommand_t* lastcmd;
}; // class OptimizedBatch

} // namespace oracle
} // namespace fatalmind
#endif
